/***********************************************************************
* ------------------------------------------
* Bustedware LLC - 2015
* https://www.bustedware.com
* ------------------------------------------
* 
* This file is free to distribute
* Please see LICENSE_1_0.txt included with this release
* 
//*********************************************************************/

#ifndef BWINDOW_H
#define BWINDOW_H

#ifdef _WIN32
#include <Windows.h>
#elif __linux__
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>
#include <X11/Xatom.h>
#include <stdlib.h>

struct screen_state {
	int 				rate;
	Rotation            rotation;
	SizeID              size_id;
};

struct screen_configuration {
	int 	m_width;
	int 	m_height;
	int 	n_frequencies;
	int* 	m_frequencies;
};
#endif
#include <string.h>
#include <iostream>
#include <map>
#include <string>
#include "BWStrings.h"
#include "BWindowManager.h"
using namespace std;

typedef struct {
	int		xcenter;
	int		ycenter;
	int		xborder;
	int		yborder;
	int 	xpos;
	int		ypos;
	int		width;
	int		height;
	int		inner_width;
	int		inner_height;
	int		top;
	int		left;
} BWindowDetail;

class BWindow
{
public:
	BWindow(const char* title, int width = 320, int height = 200, const char* mWindowName = "main_window");
	BWindow(BWindow* parent, int xPos, int yPos, int width = 320, int height = 200, const char* mWindowName = "main_window");
	~BWindow();
	
	int MsgBox(const char* message, const char* caption, unsigned int uType = 0);
	int MsgBox(const int number, const char* caption, unsigned int uType = 0);
	
	const char* GetTitle()const;
	const std::string& GetWindowName()const;
	int GetWidth()const;
	int GetHeight()const;
	bool AddChild(BWindow* child);
	bool RemoveChild(BWindow* child);
	
	void SetWidth(int width);
	void SetHeight(int height);
	void SetStyle(int cStyle);
	void SetEnabled(bool enabled);
	void SetVisible(bool visible);
	void SetPosition(int x, int y);
	void SetFullscreen(bool fullscreen);
	void SetCursorPosition(int x, int y);
	void SetBackgroundColor(int r, int g, int b);

	bool InitWindow();
	virtual void Destroy();
	BWindowDetail GetWindowDetails();
	bool operator==(BWindow* window);
	virtual bool HasNextWindowEvent(bool* kill_signal);
	
	const char* OpenFileDialog()const;
	
#ifdef _WIN32
	const HWND& GetHWND()const;
	const HINSTANCE& GetINST()const;
	void SetIconBig(HICON icon);
	void SetIconSmall(HICON icon);
	void SetMaxButtonEnabled(bool enabled);
	void SetMinButtonEnabled(bool enabled);
	void SetCursor(HCURSOR cursor);
	void SetWindowStyle(DWORD style);
	void SetWindowExStyle(DWORD style = 0);
	void SetWindowExtraBytes(int bytesExtra);
	void SetWindowExtraInstanceBytes(int extraBytes);
	const char* OpenFileDialog(OPENFILENAME& ofn)const;
	virtual LRESULT WndProc(UINT message, WPARAM wParam, LPARAM lParam);
	static LRESULT CALLBACK StaticProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
#elif __linux__
	void SetupWindowAttributes(unsigned int classType, Visual* visual, unsigned long valuemask, XSetWindowAttributes* attributes);
	Window GetWindow();
	Display* GetDisplay();
	int GetDefaultScreen();
#endif

protected:
	void findCenterWindow();

#ifdef _WIN32
	HWND mHwnd;
#elif __linux__
	Display* mDisplay;
	Window mWindow;
	int mDefaultScreen;
	bool isInitialized;
	bool HasNextWindowEvent(XEvent* e, bool* kill_signal);
#endif

	bool mKillSignal;
	bool mFullscreen;
	BWindow* myParent;
	BWindowDetail winDetail;	
	map<BWindow*, BWindow*> myChildren;
	
private:
	void SetDefaults();
		
#ifdef _WIN32
	DWORD mWinStyle, mWinExStyle;
	HINSTANCE mInst;
	WNDCLASSEX mWndClass;
	MSG mWinMsg;
	bool isMaxButtonEnabled;
	bool isMinButtonEnabled;
	void SetDefaultOpenFileName(OPENFILENAME& ofn)const;
#elif __linux__
	int nScreenConfigurations;
	void getScreenConfigurations();
	void getScreenDetails();
	void changeScreenResolution(int width, int height);
	int ProcessNonMaskedEvents(bool* kill_signal);
	screen_configuration* mScreenConfigurations;
	screen_state mCurrentScreenState;

	Visual* mVisual;
	unsigned long mBackgroundColor;
	unsigned long mClassType;
	unsigned long mValueMask;
	XSetWindowAttributes* mAttributes;
	Atom wmDeleteWindow;
	
	BWindow* mMsgBox;
#endif

	char* mTitle;
	bool isEnabled;
	bool isVisible;
	string mWindowName;
};

#endif
